﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Arction.LightningChartUltimate;
using Arction.LightningChartUltimate.Axes;
using Arction.LightningChartUltimate.SeriesXY;
using Arction.LightningChartUltimate.Views.ViewXY;

namespace CustomXAxisLables
{
    public partial class Form1 : Form
    {
        Timer timer = new Timer();
        public Form1()
        {
            InitializeComponent();
            InitChart();

            timer.Interval = 100;
            timer.Tick += timer_Tick;
            timer.Start();
        }

        private void InitChart()
        {
            chart.BeginUpdate();

            //차트 배경
            chart.Background.Color = Color.Transparent;
            chart.Background.GradientColor = Color.Transparent;
            chart.Background.Style = RectFillStyle.None;

            chart.Title.Visible = false;
            chart.RenderOptions.AntiAliasLevel = 0; // Disable hw anti-aliasing.

            ViewXY chartView = chart.ViewXY;

            chartView.AxisLayout.YAxesLayout = YAxesLayout.Stacked;

            chartView.ZoomPanOptions.LeftMouseButtonAction = MouseButtonAction.None;
            chartView.ZoomPanOptions.RightMouseButtonAction = MouseButtonAction.None;

            chartView.GraphBorderColor = Color.Black;
            chartView.GraphBackground.Color = Color.Transparent;
            chartView.GraphBackground.GradientColor = Color.Transparent;
            chartView.GraphBackground.Style = RectFillStyle.None;

            chartView.LegendBox.Position = LegendBoxPosition.TopLeft;
            chartView.LegendBox.Visible = false;
            chartView.DropOldSeriesData = true;

            //Y축 위치 자동 설정
            chartView.AxisLayout.AutoAdjustMargins = false;
            //Y축 타이틀 자동 설정
            chartView.AxisLayout.YAxisTitleAutoPlacement = false;
            chartView.AxisLayout.YAxisAutoPlacement = YAxisAutoPlacement.Off;
            chartView.AxisLayout.YAxesLayout = YAxesLayout.Stacked;

            chartView.AxisLayout.XAxisAutoPlacement = XAxisAutoPlacement.AllBottom;
            chartView.AxisLayout.XAxisTitleAutoPlacement = false;


            chartView.XAxes.Clear();

            AddXAxis(chart);
            chartView.YAxes[0].SetRange(0, 100);
            AddSeries();
            
            chart.EndUpdate();
            CreateCustomLabels();
        }
        LabelInfo[] labels;

        struct LabelInfo
        {
            public PointF Position;
            public string Text;
        }

        private void CreateCustomLabels()
        {
            ViewXY chartView = this.chart.ViewXY;
            AxisX axisX = chartView.XAxes[0];
            AxisY axisY = chartView.YAxes[0];

            float minX = axisX.ValueToCoord(axisX.Minimum) - 10;
            float minY = axisY.ValueToCoord(axisY.Minimum) + 5;
            float maxX = axisX.ValueToCoord(axisX.Maximum) - 10;
            float maxY = axisY.ValueToCoord(axisY.Maximum) + 5;

            //전체 시간
            int totalViewSec = (int)(axisX.Maximum - axisX.Minimum);
            int labelCount = 11;

            int xStep = (int)(maxX - minX) / (labelCount - 1);
            int xPos = (int)minX;
            int yPos = (int)minY;
            

            labels = new LabelInfo[labelCount];
            
            for (int i = 0; i < labelCount; i++)
            {
                TimeSpan ts = new TimeSpan(0, 0, i);
                string text = ts.ToString(@"mm\:ss");
                Point position = new Point(xPos, yPos);
                
                labels[i] = new LabelInfo() { Position = position, Text = text };

                xPos += xStep;
            }
        }
        
        private void AddXAxis(LightningChartUltimate chart)
        {
            ViewXY chartView = chart.ViewXY;

            AxisX xAxis = new AxisX(chartView);

            xAxis.ScrollPosition = 0;
            xAxis.ScrollMode = XAxisScrollMode.Scrolling;
            xAxis.SweepingGap = 1;

            xAxis.Title.Visible = false;
            xAxis.SetRange(0, 10);
            xAxis.LabelsColor = Color.Black;
            xAxis.AxisColor = Color.Black;
            xAxis.AxisThickness = 1;
            xAxis.MouseInteraction = false;
            xAxis.AutoFormatLabels = false;
            //xAxis.ValueType = AxisValueType.Time;
            xAxis.MajorDivTickStyle.Visible = false;
            xAxis.MinorDivTickStyle.Visible = false;
            //xAxis.LabelsVisible = false;
            //xAxis.FormatValueLabel += xAxis_FormatValueLabel;

            chartView.XAxes.Add(xAxis);
        }

        public void AddSeries()
        {
            ViewXY chartView = this.chart.ViewXY;

            //Y축의 맨마지막 지정
            SampleDataSeries series = new SampleDataSeries(chartView, chartView.XAxes[0], chartView.YAxes[0]);
            
            series.LineStyle.Color = Color.Red;
            series.LineStyle.Width = 1;
            series.LineStyle.AntiAliasing = LineAntialias.None;
            series.SampleFormat = SampleFormat.SingleFloat;
            series.FirstSampleTimeStamp = 0;
            series.SamplingFrequency = sampleFreq;
            series.PointsOptimization = PointsRenderOptimization.Pixels;
            series.ScrollingStabilizing = true;
            series.ScrollModePointsKeepLevel = 1;

            series.MouseInteraction = false;
            series.MouseHighlight = MouseOverHighlight.None;
            chartView.SampleDataSeries.Add(series);

        }
        string xAxis_FormatValueLabel(AxisBase axis, double value)
        {
            AxisX x = axis as AxisX;
            float location = x.ValueToCoord(value);
            //return location.ToString();
            return "";
        }

        Random rand = new Random();
        int pointCount = 0;
        double sampleFreq = 10;
        void timer_Tick(object sender, EventArgs e)
        {
            this.Invoke(new MethodInvoker(delegate {
                var series = this.chart.ViewXY.SampleDataSeries[0];
                series.AddSamples(new float[] { rand.Next(0, 100) }, true);
                pointCount++;
                this.chart.ViewXY.XAxes[0].ScrollPosition = (pointCount - 1) / sampleFreq;
            }));
        }

        private void chart_AfterRendering(LightningChartUltimate chart)
        {
            if (labels == null)
                return;

            using (Graphics g = chart.CreateGraphics())
            {
                foreach (var item in labels)
                {
                    g.DrawString(item.Text, this.Font, Brushes.Black, item.Position);
                }
            }
        }
    }
}
